use clap::{Arg, Command};

pub mod asotr_data {
    use std::{fs::File, io::Read};
    use byteorder::{LittleEndian, ReadBytesExt};
    use chrono::{DateTime, Utc};
    use std::time::{SystemTime, UNIX_EPOCH, Duration};
    use strum::FromRepr;
    
    #[derive(Debug, FromRepr, PartialEq)]
    enum AsotrDataType {
        Temp = 1,
        Pow = 2,
        TempSet = 6,
    }

    struct AsotrDataDesc {
        time_s: u64,
        time_mks: u32,
        datetime: String,
        date: String,
        time: String,
        data_type: AsotrDataType,
        kit: u8,
    }
    
    impl AsotrDataDesc {
        pub fn new(time_s: u64, time_mks: u32, 
                   datetime: String, date: String, time: String, 
                   data_type: AsotrDataType, kit: u8) -> AsotrDataDesc {
            AsotrDataDesc { time_s, time_mks, datetime, date, time, data_type, kit}
        }
    }

    fn parse_data_f32(buf: Vec<u8>) -> Result<[f32; 6], String> {
        let mut data = &buf[..];
        let mut ch: [f32; 6] = [0.0; 6];

        for i in 0..6 {
            ch[i] = match data.read_f32::<LittleEndian>() {
                Ok(val) => val,
                Err(msg) => { 
                    return Err(format!("Error parsing file: failed parsing float32 data: {}", msg)); }
            }
        }

        return Ok(ch);
    }

    fn parse_data_u16(buf: Vec<u8>) -> Result<[u16; 6], String> {
        let mut data = &buf[..];
        let mut ch: [u16; 6] = [0; 6];

        for i in 0..6 {
            ch[i] = match data.read_u16::<LittleEndian>() {
                Ok(val) => val,
                Err(msg) => { 
                    return Err(format!("Error parsing file: failed parsing float32 data: {}", msg)); }
            }
        }

        return Ok(ch);
    }


    fn parse_filename(filename_full: String) -> Result<(AsotrDataDesc, String), String> {
        let mut filename = String::new();

        match filename_full.rfind('/') {
            Some(val) => { filename = (filename_full[val+1..filename_full.len()]).to_string(); }
            _ => { filename = filename_full.clone();  }
        }

        if filename.len() != 32 { return Err(format!("Error parsing filename: Unsupported file ({})", filename)); }

        let time_unix_ = filename[0..10].parse::<u64>();
        let time_unix = match &time_unix_ {
            Ok(data) => data,
            Err(msg) => { return Err(format!("Error parsing filename: expected digits in timestamp sec part ({})", msg));  }
        };

        let data_type_ = filename[22..24].parse::<u8>();
        let data_type_u8 = match &data_type_ {
            Ok(data) => data,
            Err(msg) => { return Err(format!("Error parsing filename: expected digits in data type part ({})", msg));  }
        };

        if *data_type_u8 == 1 || *data_type_u8 == 2 || *data_type_u8 == 6 { }
        else {
            return Err("unsupported file type! Parser supports data types: .data01.asotr01(02), data02.asotr01(02), data06.asotr01(02)".to_string());
        }

        let data_type = match AsotrDataType::from_repr(*data_type_u8 as usize) {
            Some(value) => value,
            _ => return Err(format!("Error parsing filename d: expected digits in data type part"))
        };

        let _kit = filename[30..32].parse::<u8>();
        let kit = match &_kit {
            Ok(data) => data,
            Err(msg) => { return Err(format!("Error parsing filename: expected digits in asotr kit part ({})", msg));  }
        };

        let _time_str_mks = filename[11..17].parse::<u32>();
        let time_mks = match &_time_str_mks {
            Ok(data) => data,
            Err(msg) => { return Err(format!("Error parsing filename: expected digits in timestamp mks part ({})", msg));  }
        };
            
        let time: SystemTime = UNIX_EPOCH + Duration::from_secs(*time_unix);
        let date_time = DateTime::<Utc>::from(time);
        let date_s = date_time.format("%d.%m.%Y").to_string();
        let time_s = date_time.format("%H:%M:%S").to_string();

        let asotrDataHead = AsotrDataDesc::new(*time_unix, *time_mks, 
                                           format!("{} {}", date_s, time_s),
                                           date_s, time_s, data_type, *kit);

        return Ok((asotrDataHead, filename));
    }

    pub fn parse_data(filename_full: String) -> Result<String, String> {
        let ch_u16: [u16; 6];
        let ch_f32: [f32; 6];
        
        let (asotr_head, filename) = parse_filename(filename_full.clone())?;

            let mut buf = Vec::new();
            let mut out = String::new();
            
            
            let mut data = match File::open(filename_full.clone())
            {
                Ok(file) => file,
                Err(msg) => { return Err(format!("Error opening data file {}: {}", filename, msg)) }
            };

            match data.read_to_end(&mut buf) {
                Ok(stat) => stat,
                Err(msg) => { return Err(format!("Error reading data file {}: {}", filename, msg)) }
            };


            out.push_str(&asotr_head.time_s.to_string());
            out.push(';');
            out.push_str(&asotr_head.date);
            out.push(';');
            out.push_str(&asotr_head.time);
            out.push(';');

            if asotr_head.data_type == AsotrDataType::Temp || 
                asotr_head.data_type == AsotrDataType::TempSet {
                ch_f32 = parse_data_f32(buf)?;
                for elem in ch_f32 { 
                    out.push_str(&elem.to_string());  
                    out.push(';');
                }
            }
            else if asotr_head.data_type == AsotrDataType::Pow {
                ch_u16 = parse_data_u16(buf)?;
                for elem in ch_u16 { 
                    out.push_str(&elem.to_string());  
                    out.push(';');
                }
            }
            
            out.remove(out.len() - 1);

            return Ok(out);
    }

}


fn main() {
    use crate::asotr_data::*; 

    let arguments = Command::new("asotr_csv")
        .version("0.1.0")
        .author("Danila Gamkov <danila_gamkov@cosmos.ru>")
        .about("raw data ASOTR MVN parser")
        .arg(
            Arg::new("file")
            .short('f')
            .long("filename")
            .help("input ASOTR MVN data (.data01.asotr01(02), .data02.asotr01(02), .data06.asotr01(02))")
            .required(true),
            )
        .get_matches();

    let filename: &String = match arguments.get_one::<String>("file") {
        Some(fname) => fname,
        _ => { println!("Error: wrong argument!"); return;  }
    };

    let s = match parse_data(filename.clone()) {
    Ok(elem) => elem,
    Err(msg) => { println!("{}", msg); return; }
    };
       
    println!("{}", s);
}






